/*
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "notifymanager.h"

#ifdef __cplusplus
extern "C"
{
#endif
#include <dconf/dconf.h>
#ifdef __cplusplus
}
#endif

notifyManager::notifyManager(QObject *parent) : QObject(parent)
{
    m_pTopWidget = new topTransparentWidget(this);
    m_psqlInfoData = new sqlInfoData(this);
    connect(m_pTopWidget, &topTransparentWidget::notificationClosed, this, &notifyManager::popupItemWidgetDismissed);
    connect(m_pTopWidget, &topTransparentWidget::actionInvoked, this, &notifyManager::popupItemWidgetActionInvoked);
    registerAsService();
    initGsettingValue();
}

notifyManager::~notifyManager()
{
    if (m_pNodisturbModeGsetting) {
        delete m_pNodisturbModeGsetting;
        m_pNodisturbModeGsetting = NULL;
    }
    if (m_pPopupWidgetModeGsetting) {
        delete m_pPopupWidgetModeGsetting;
        m_pPopupWidgetModeGsetting = NULL;
    }
    if (m_pControlNotifyMainGsetting) {
        delete m_pControlNotifyMainGsetting;
        m_pControlNotifyMainGsetting = NULL;
    }
}

void notifyManager::CloseNotification(uint id)
{
    emit m_pTopWidget->closePopupWidget(id);
    emit NotificationClosed(id, notifyManager::ClosedByApp);
    nextShowAction();
    return;
}

QStringList notifyManager::GetCapabilities()
{
    QStringList result;
    result << "action-icons"
           << "actions"
           << "body"
           << "body-hyperlinks"
           << "body-markup";
    return result;
}

void notifyManager::GetServerInformation(QString &name, QString &vendor, QString &version, QString &specVersion)
{
    QString serverVersion = "UKUI " + QString::number(SERVERVERSION, 10, 1);
    QString protocolVersion = "Desktop Notifications Specification " + QString::number(PROTOCOLVERSION, 10, 1);
    name = QString("ukui-notification-daemon");
    vendor = QString("Kylin OS");
    version = serverVersion;
    specVersion = protocolVersion;
    return;
}

uint notifyManager::Notify(
    const QString &appName,
    uint replacesId,
    const QString &appIcon,
    const QString &summary,
    const QString &body,
    const QStringList &actions,
    const QVariantMap hints,
    int expireTimeout)
{
    ++m_notifyNum;
#ifdef QT_DEBUG
    qInfo() << "a new Notify1:"
            << "appName2:" + appName << "replaceID3:" + QString::number(replacesId) << "appIcon:" + appIcon
            << "summary:" + summary << "body:" + body << "actions:" << actions << "hints:" << hints
            << "expireTimeout:" << expireTimeout;
#endif

    // 免打扰模式判断
    if (m_bNodisturbMode) {
        qInfo() << "Undisturbed mode，return";
        return 0;
    }

    if (PLATFORM::g_platformType == PLATFORM::V101) {
        // 控制面板通知总开关
        if (!m_bNotifyMainSwitch) {
            qInfo() << "Mask all messages,No pop-up window";
            return 0;
        }
        // 对应应用通知是否弹窗
        bool enable = getControlCentorAppNotify(appName);
        if (!enable) {
            qInfo() << "Mask " << appName << " messages,No pop-up window";
            return 0;
        }
    }
    counter++;
    notifyReceiveInfo *notifyInfo = new notifyReceiveInfo(
        appName,
        QString::number(counter),
        appIcon,
        summary,
        body,
        actions,
        hints,
        QString::number(QDateTime::currentMSecsSinceEpoch()),
        QString::number(replacesId),
        QString::number(expireTimeout),
        this);
    m_psqlInfoData->addOne(notifyInfo);
    if (PLATFORM::g_platformType == PLATFORM::V101) {
        // 加入弹窗声音
        appNotifySound();
    }
    if (m_pTopWidget->isPopupExist(replacesId)) { //替换通知弹窗内容
        --m_notifyNum;
        m_pTopWidget->replacePopupItemWidget(replacesId, notifyInfo);
    } else { //新建通知弹窗
        // 单弹窗模式 多弹窗模式
        if (m_bPopupWidgetModeStatus) { //单弹窗模式
            m_pTopWidget->addEntryInfo(notifyInfo);
            if (!m_pTopWidget->isVisible())
                m_pTopWidget->consumeEntities();
        } else { //多弹窗模式
            m_pTopWidget->AddPopupItemWidget(notifyInfo);
            if (!m_pTopWidget->isVisible())
                m_pTopWidget->show();
        }
    }
    return replacesId == 0 ? counter : replacesId;
}
/* 针对Python的通知接口 */
uint notifyManager::NotifyPy(
    const QString &appName,
    int replacesId,
    const QString &appIcon,
    const QString &summary,
    const QString &body,
    const QVariantList &actions,
    const QVariantMap hints,
    int expireTimeout)
{
    // Python的列表[]对应C++中的QVariantList类型，需要将QVariantList转换成QStringList类型后Notify()才可使用
    QStringList actionsList;
    for (int i = 0; i < actions.size(); i++) {
        actionsList.append(actions.at(i).toString());
    }
    return Notify(appName, replacesId, appIcon, summary, body, actionsList, hints, expireTimeout);
}
/* 简单通知接口：无action和hints 字段 */
uint notifyManager::NotifySimple(
    const QString &appName,
    uint replacesId,
    const QString &appIcon,
    const QString &summary,
    const QString &body,
    int expireTimeout)
{
    const QStringList actions;
    const QVariantMap hints;
    return (Notify(appName, replacesId, appIcon, summary, body, actions, hints, expireTimeout));
}

/* 注册 freedesktop Notifications 服务*/
void notifyManager::registerAsService()
{
    QDBusConnection connection = QDBusConnection::sessionBus();
    connection.interface()->registerService(
        NotificationsDBusService,
        QDBusConnectionInterface::ReplaceExistingService,
        QDBusConnectionInterface::AllowReplacement);
    connection.registerObject(NotificationsDBusPath, this);
}
/* 显示下一条通知：单弹窗模式下会调用，当上一条通知超时退出后，显示队列中的下一条 */
void notifyManager::nextShowAction()
{
    if (m_bPopupWidgetModeStatus && !m_pTopWidget->isVisible()) {
        m_pTopWidget->consumeEntities();
    } else if (0 == m_pTopWidget->m_popWidgetqueue.count()) {
        quitHandle();
    }
    return;
}

/* 初始化 弹窗模式、弹窗禁用 Gsetting */
void notifyManager::initGsettingValue()
{
    // 初始化免打扰
    const QByteArray id(UKUI_VOLUME_BRIGHTNESS_GSETTING_ID);
    if (QGSettings::isSchemaInstalled(id)) {
        m_pNodisturbModeGsetting = new QGSettings(id);
        connect(m_pNodisturbModeGsetting, &QGSettings::changed, this, [=](QString key) {
            if (key == KYLIN_DISTURB_GSETTING_VALUE_NOTIFYCLOSE) {
                m_bNodisturbMode = m_pNodisturbModeGsetting->get(KYLIN_DISTURB_GSETTING_VALUE_NOTIFYCLOSE).toBool();
            }
        });
        if (m_pNodisturbModeGsetting != nullptr
            && m_pNodisturbModeGsetting->keys().contains(KYLIN_DISTURB_GSETTING_VALUE_NOTIFYCLOSE))
            m_bNodisturbMode = m_pNodisturbModeGsetting->get(KYLIN_DISTURB_GSETTING_VALUE_NOTIFYCLOSE).toBool();
    }
    // 初始化弹窗模式
    const QByteArray id_2(UKUI_NOTIFICATION_DEMO_GSETTING_ID);
    if (QGSettings::isSchemaInstalled(id_2)) {
        m_pPopupWidgetModeGsetting = new QGSettings(id_2);
        connect(m_pPopupWidgetModeGsetting, &QGSettings::changed, this, [=](QString key) {
            if (key == KYLIN_NOTIFICATION_DEMO_CLOSE_MODE_KEY) {
                m_bPopupWidgetModeStatus
                    = m_pPopupWidgetModeGsetting->get(KYLIN_NOTIFICATION_DEMO_CLOSE_MODE_KEY).toBool();
            }
        });
        if (m_pPopupWidgetModeGsetting != nullptr
            && m_pPopupWidgetModeGsetting->keys().contains(KYLIN_NOTIFICATION_DEMO_CLOSE_MODE_KEY))
            m_bPopupWidgetModeStatus = m_pPopupWidgetModeGsetting->get(KYLIN_NOTIFICATION_DEMO_CLOSE_MODE_KEY).toBool();
    }

    if (PLATFORM::g_platformType == PLATFORM::V101) {
        // 初始化通知弹窗总开关
        const QByteArray id_3(CONTROL_CENTER_GSETTING_NOTIFY_MAIN_SWITCH);
        if (QGSettings::isSchemaInstalled(id_3)) {
            m_pControlNotifyMainGsetting = new QGSettings(id_3);
            if (m_pControlNotifyMainGsetting != nullptr
                && m_pControlNotifyMainGsetting->keys().contains(CONTROL_GSETTING_NOTIFY_MAIN_KEY)) {
                m_bNotifyMainSwitch = m_pControlNotifyMainGsetting->get(CONTROL_GSETTING_NOTIFY_MAIN_KEY).toBool();
                connect(m_pControlNotifyMainGsetting, &QGSettings::changed, this, [=](QString key) {
                    if (key == CONTROL_GSETTING_NOTIFY_MAIN_KEY) {
                        m_bNotifyMainSwitch
                            = m_pControlNotifyMainGsetting->get(CONTROL_GSETTING_NOTIFY_MAIN_KEY).toBool();
                    }
                });
            }
        }
    }
    return;
}

/* 获取系统当前声效列表：四类声效 system-setting、 window-close、alert-sound、volume-changed*/
QList<char *> notifyManager::listExistsPath()
{
    char **childs;
    int len;
    DConfClient *client = dconf_client_new();
    childs = dconf_client_list(client, KEYBINDINGS_CUSTOM_DIR, &len);
    g_object_unref(client);

    QList<char *> vals;
    for (int i = 0; childs[i] != NULL; i++) {
        if (dconf_is_rel_dir(childs[i], NULL)) {
            char *val = g_strdup(childs[i]);
            vals.append(val);
        }
    }
    g_strfreev(childs);
    return vals;
}
/* 播放通知提示音 */
void notifyManager::appNotifySound()
{
    ca_context_create(&m_pCaContext);
    //获取系统的通知声效
    gint retval;
    const gchar *desc = "alert sound";
    QString filenameStr;
    QList<char *> existsPath = this->listExistsPath();
    for (char *path : existsPath) {
        char *prepath = QString(KEYBINDINGS_CUSTOM_DIR).toLatin1().data();
        char *allpath = strcat(prepath, path);
        const QByteArray ba(KEYBINDINGS_CUSTOM_SCHEMA);
        const QByteArray bba(allpath);
        if (QGSettings::isSchemaInstalled(ba)) {
            QGSettings *settings = new QGSettings(ba, bba);
            filenameStr = settings->get(FILENAME_KEY).toString();
            QString nameStr = settings->get(NAME_KEY).toString();
            qInfo() << filenameStr << nameStr;
            if (nameStr == "alert-sound") {
                break;
            }
            delete settings;
            settings = nullptr;
        }
    }
    //获取通知声效开关状态
    bool b_soundSwitchStatus = true;
    const QByteArray soundSwitch(CONTROL_PATH_SOUNDS_SCHEMA);
    if (QGSettings::isSchemaInstalled(soundSwitch)) {
        QGSettings *soundSwitchSetting = new QGSettings(soundSwitch);
        b_soundSwitchStatus = soundSwitchSetting->get(CONTROL_EVENT_SOUNDS_KEY).toBool();
        delete soundSwitchSetting;
        soundSwitchSetting = NULL;
    }
    //播放通知声音
    const QByteArray text = filenameStr.toLocal8Bit();
    const gchar *id = text.data();
    const gchar *eventId = id;
    if (desc && b_soundSwitchStatus) {
        retval = ca_context_play(m_pCaContext, 0, CA_PROP_EVENT_ID, eventId, CA_PROP_EVENT_DESCRIPTION, desc, NULL);
        qInfo() << desc << id << retval;
    }
}

/* 获取应用通知开关状态 */
bool notifyManager::getControlCentorAppNotify(QString appName)
{
    if (appName.isEmpty()) {
        return false;
    }
    // 初始化控制面板对于通知开关读取
    const QByteArray id_3(CONTROL_CENTER_GSETTING_PATH);
    if (QGSettings::isSchemaInstalled(id_3)) {
        QString dynamicPath = QString("%1%2/").arg(CONTROL_CERTER_DYNAMIC_GSETTING_PATH).arg(QString(appName));
        const QByteArray id_4(dynamicPath.toUtf8().data());
        m_pAppNotifyGseting = new QGSettings(id_3, id_4, this);
        bool status = m_pAppNotifyGseting->get(CONTROL_CENTER_GSETTING_NOTIFYCATION).toBool();
        delete m_pAppNotifyGseting;
        m_pAppNotifyGseting = NULL;
        return status;
    } else {
        return false;
    }
}

/* 点击关闭按钮、消息体、超时等 弹窗退出 */
void notifyManager::popupItemWidgetDismissed(uint id, uint closeReson)
{
    --m_notifyNum;
    emit NotificationClosed(id, closeReson);
    nextShowAction();
    return;
}

/* 点击弹窗的动作按钮退出 */
void notifyManager::popupItemWidgetActionInvoked(uint id, QString actionKey)
{
    --m_notifyNum;
    emit ActionInvoked(id, actionKey);
    nextShowAction();
    return;
}

void notifyManager::quitHandle()
{
    if (0 == m_pTopWidget->m_popWidgetqueue.count() && m_notifyNum <= 0) {
        qApp->quit();
    }
}
