/*
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "toptransparentwidget.h"
#include "notifyreceiveinfo.h"
#include <QPainterPath>
#include <QDBusMessage>
#include <QDBusConnection>
#include <QTime>
#include <QGraphicsDropShadowEffect>
#include "commondefinition.h"

/* 最外层透明窗口四周各1xp阴影 */
#define TOP_SPACING 1
#define BOTTON_SPACING 1
#define LEFT_SPACING 1
#define RIGHT_SPACING 1

topTransparentWidget::topTransparentWidget(QObject *parent)
{
    Q_UNUSED(parent);
    /* 初始化UI布局和窗口属性 */
    initUiLayout();
    /* 初始化任务栏相关Gsettings，监听任务栏高度和方向 */
    initPanelSite();
    /* 实例化屏幕信息类，监听屏幕变化 */
    m_pSreenInfo = new adaptScreenInfo(this);
    connect(m_pSreenInfo, &adaptScreenInfo::updataTopWidgetSite, this, [=]() { setNotifyPopWidgetSite(); });
    /* 设置通知弹窗的位置 */
    setNotifyPopWidgetSite();
    /* 初始化 通知与侧边栏联动动画 */
    if (PLATFORM::g_platformType == PLATFORM::Intel) {
        QDBusConnection::sessionBus().connect(
            QString(""),
            QString("/org/ukui/Sidebar"),
            "org.ukui.Sidebar",
            "animationAction",
            this,
            SLOT(animationActionSlots(uint, int)));
        m_posAnimation = new QPropertyAnimation(this, "pos");
    }
    /* 绑定窗口关闭信号 */
    connect(this, &topTransparentWidget::closePopupWidget, this, &topTransparentWidget::moveAllpopWidgetSiteAccordId);
    return;
}

topTransparentWidget::~topTransparentWidget()
{
    if (m_pPanelSetting) {
        delete m_pPanelSetting;
        m_pPanelSetting = NULL;
    }
}

/* 初始化UI布局和窗口属性 */
void topTransparentWidget::initUiLayout()
{
    //初始化布局
    m_pMainLayout = new QVBoxLayout(this);
    m_pMainLayout->setContentsMargins(0, 0, 0, 0);
    m_pMainLayout->setDirection(QBoxLayout::BottomToTop);
    m_pMainLayout->setSpacing(5);
    this->setContentsMargins(LEFT_SPACING, TOP_SPACING, RIGHT_SPACING, BOTTON_SPACING);
    if (PLATFORM::g_platformType == PLATFORM::V101) {
        this->setFixedWidth(MASTER::topWidgetWidth + LEFT_SPACING + RIGHT_SPACING);
    } else if(PLATFORM::g_platformType == PLATFORM::Intel) {
        this->setFixedWidth(INTER::topWidgetWidth + LEFT_SPACING + RIGHT_SPACING);
    }

    this->setLayout(m_pMainLayout);
    //设置窗口属性
    setWidgetAttribute();
}

/* 设置窗口属性 */
void topTransparentWidget::setWidgetAttribute()
{
    setWindowFlags(
        Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint); //无边框、禁止拖动、禁止改变大小、不受窗管管理
    setAttribute(Qt::WA_TranslucentBackground);                    //透明背景
    setAttribute(Qt::WA_AlwaysShowToolTips);
    return;
}

/* 设置窗口位置 */
void topTransparentWidget::setWidgetPos(int x, int y)
{
    qInfo() << "set pop-up window X:" << x;
    qInfo() << "set pop-up window Y:" << y;
    this->move(x, y);
    return;
}

/* 添加弹窗到TopWidget中 */
void topTransparentWidget::AddPopupItemWidget(notifyReceiveInfo *entryInfo)
{
    if (entryInfo == nullptr) {
        return;
    }
    popupItemWidget *popw = new popupItemWidget(this, entryInfo);

    if (m_popWidgetqueue.count() == 0) {
        m_ListWidgetHeight = 0;
    }

    //判断是否设置定时，若设置定时，走独立定时分支。若不设置定时，累加定时器
    if (entryInfo->timeout().toInt() == -1) {  //默认驻留时间
        if(m_popWidgetqueue.count() >= m_fixNotifyNum){
           popw->m_poutTimer->setInterval(g_defaultTimer * (m_popWidgetqueue.count() - m_fixNotifyNum + 1));
        }
        else {
            popw->m_poutTimer->setInterval(g_defaultTimer * (m_popWidgetqueue.count() + 1));
        }
    } else if (entryInfo->timeout().toInt() == 0) { //弹窗常驻
        ++m_fixNotifyNum;
        popw->m_poutTimer->blockSignals(true);
        popw->m_quitTimer->blockSignals(true);
    } else { //应用自定义驻留时间
        popw->m_poutTimer->setInterval(entryInfo->timeout().toInt() * 1);
    }

    /* 绑定弹窗一系列动作 */
    //弹窗超时退出
    connect(popw, &popupItemWidget::timeout, this, &topTransparentWidget::startPopWidgetHideAnimation);
    //点击弹窗消息体退出
    connect(popw, &popupItemWidget::clickedNotifyBody, this, &topTransparentWidget::startPopWidgetHideAnimation);
    //点击弹窗右上角关闭按钮，退出
    connect(popw, &popupItemWidget::clickedCloseBtn, this, &topTransparentWidget::startPopWidgetHideAnimation);
    //点击执行默认动作信号，退出
    connect(popw, &popupItemWidget::clickedDefaultAction, this, &topTransparentWidget::startPopWidgetHideAnimation);
    //点击动作按钮，退出
    connect(popw, &popupItemWidget::clickedActionButton, this, &topTransparentWidget::startPopWidgetHideAnimation);
    //隐藏动画完成，退出
    connect(popw, &popupItemWidget::outAnimationFinishSignal, this, &topTransparentWidget::hideAnimationFinishSlots);
    connect(popw, &popupItemWidget::animationAction, this, &topTransparentWidget::TransformGroundGlassAreaSlots);

    popw->setEntryData(entryInfo); //填充一条通知
    m_popWidgetqueue.append(popw);

    //布局
    if (m_popWidgetqueue.count() == 1) {
        m_ListWidgetHeight += popw->height();
        this->setFixedHeight(m_ListWidgetHeight + TOP_SPACING + BOTTON_SPACING);
        m_pMainLayout->addWidget(popw);
        popw->setGeometry(0, 0, popw->width(), popw->height());
        this->update();
   } else {
        if(PLATFORM::g_platformType == PLATFORM::V101){
            if (m_ListWidgetHeight < m_pSreenInfo->m_screenHeight - MASTER::topWidgeReservedHeight) {
                m_ListWidgetHeight += popw->height() + MASTER::popupWidgetSpaceHeight;
                this->setFixedHeight(m_ListWidgetHeight + TOP_SPACING + BOTTON_SPACING);
                m_pMainLayout->addWidget(popw);
                popw->setGeometry(0, 0, popw->width(), popw->height());
                this->update();
            } else { //如果超出屏幕高度，则加入等待队列
                m_pWaitingQueue.append(entryInfo);
                m_popWidgetqueue.removeAt(m_popWidgetqueue.count() - 1);
                delete popw;
                popw = NULL;
            }
        }
        else if(PLATFORM::g_platformType == PLATFORM::Intel){
            if (m_ListWidgetHeight < m_pSreenInfo->m_screenHeight - INTER::topWidgeReservedHeight) {
               m_ListWidgetHeight += popw->height() + INTER::popupWidgetSpaceHeight;
               this->setFixedHeight(m_ListWidgetHeight + TOP_SPACING + BOTTON_SPACING);
               m_pMainLayout->addWidget(popw);
               popw->setGeometry(0, 0, popw->width(), popw->height());
               this->update();
           } else {
               m_pWaitingQueue.append(entryInfo);
               m_popWidgetqueue.removeAt(m_popWidgetqueue.count() - 1);
               delete popw;
               popw = NULL;
           }
        }
    }
    return;
}

/* 替换通知弹窗的内容 */
void topTransparentWidget::replacePopupItemWidget(uint oldInfoId, notifyReceiveInfo *replaceInfo)
{
    for (int i = 0; i < m_popWidgetqueue.size(); ++i) {
        popupItemWidget *popwidget = m_popWidgetqueue.at(i);
        if (popwidget->m_pentryInfo->id().toUInt() == oldInfoId) {
            popwidget->updateWidgetDate(replaceInfo);
            break;
        }
    }
}

/* 删除popwidget弹窗 */
void topTransparentWidget::exitPopupWidget(QWidget *w)
{
    popupItemWidget* popw = dynamic_cast<popupItemWidget *>(w);
    m_ListWidgetHeight = m_ListWidgetHeight - w->height() - MASTER::popupWidgetSpaceHeight;
    m_pMainLayout->removeWidget(popw);
    m_popWidgetqueue.removeOne(popw);
    delete popw->m_pentryInfo;
    popw = NULL;
    if (m_ListWidgetHeight <= 0) {
        this->setFixedHeight(0);
    } else {
        this->setFixedHeight(m_ListWidgetHeight + TOP_SPACING + BOTTON_SPACING);
    }
    if (0 == m_popWidgetqueue.count()) {
        this->hide();
        return;
    }
    this->update();
    return;
}

/* 通知信息加入到通知队列中（单弹窗模式中使用） */
void topTransparentWidget::addEntryInfo(notifyReceiveInfo *entryInfo)
{
    m_entities.enqueue(entryInfo);
    return;
}

/* 添加一条通知到TopWidget中显示（单弹窗模式中使用）*/
void topTransparentWidget::consumeEntities()
{
    if (!m_currentNotify.isNull()) {
        m_currentNotify->deleteLater();
        m_currentNotify = nullptr;
    }

    if (m_entities.isEmpty()) {
        m_currentNotify = nullptr;
        qApp->quit();
        return;
    }

    m_currentNotify = m_entities.dequeue();
    this->setProperty("blurRegion", QRegion(QRect(0, 0, MASTER::topWidgetWidth, this->height())));
    AddPopupItemWidget(m_currentNotify);
    this->show();
    return;
}

/* 初始化任务栏相关Gsettings，获取任务栏高度和方向 */
void topTransparentWidget::initPanelSite()
{
    if (QGSettings::isSchemaInstalled(UKUI_PANEL_SETTING))
        m_pPanelSetting = new QGSettings(UKUI_PANEL_SETTING);
    if (m_pPanelSetting != nullptr) {
        connect(m_pPanelSetting, &QGSettings::changed, this, &topTransparentWidget::panelSiteSlots);
        QStringList keys = m_pPanelSetting->keys();
        /* 获取任务栏位置 */
        if (keys.contains("panelposition")) {
            m_ipanelPosition = m_pPanelSetting->get("panelposition").toInt();
        }
        /* 获取任务栏高度 */
        if (keys.contains("panelsize")) {
            m_ipanelHeight = m_pPanelSetting->get("panelsize").toInt();
        }
    }
    return;
}

/* 设置通知弹窗的位置 */
void topTransparentWidget::setNotifyPopWidgetSite()
{
    if (PLATFORM::g_platformType == PLATFORM::V101) {
        m_iScreenXGeometry = m_pSreenInfo->m_nScreen_x;
        m_iScreenYGeometry = m_pSreenInfo->m_nScreen_y;
        m_iScreenWidth = m_pSreenInfo->m_screenWidth;
        m_iScreenHeight = m_pSreenInfo->m_screenHeight;
        switch (m_ipanelPosition) {
            case 1:
                setWidgetPos(m_iScreenXGeometry + m_iScreenWidth - MASTER::topWidgetWidth - MASTER::topWidgeMargin,
                     m_iScreenYGeometry + m_ipanelHeight + MASTER::topWidgeMargin);
                break;
            case 2:
                setWidgetPos(m_iScreenXGeometry + m_iScreenWidth - MASTER::topWidgetWidth - MASTER::topWidgeMargin,
                     m_iScreenYGeometry + MASTER::topWidgeMargin);
                break;
            case 3:
                setWidgetPos(m_iScreenXGeometry + m_iScreenWidth - MASTER::topWidgetWidth - m_ipanelHeight - MASTER::topWidgeMargin,
                     m_iScreenYGeometry + MASTER::topWidgeMargin);
                break;
            default:
                setWidgetPos(m_iScreenXGeometry + m_iScreenWidth - MASTER::topWidgetWidth - MASTER::topWidgeMargin,
                     m_iScreenYGeometry + MASTER::topWidgeMargin);
                break;
        }
    } else if (PLATFORM::g_platformType == PLATFORM::Intel) {
        //获取侧边栏显示的宽度(Intel项目中通知弹窗跟随侧边栏展开和收纳）
        int sidebarWidth = 0;
        QDBusMessage msg = QDBusMessage::createMethodCall(
            "org.ukui.Sidebar", "/org/ukui/Sidebar", "org.ukui.Sidebar", "getPresentSidebarWidth");
        QDBusMessage response = QDBusConnection::sessionBus().call(msg);
        if (response.type() == QDBusMessage::ReplyMessage) {
            sidebarWidth = response.arguments().takeFirst().toInt();
        } else {
            qCritical() << "getPresentSidebarWidth error:" << response.type();
            sidebarWidth = 0;
        }
        //获取当前屏幕旋转状态  (四种状态 "normal" "upside-down" "left" "right")
        QString rotationState = "normal";
        QDBusMessage rotationMsg = QDBusMessage::createMethodCall(
            "com.kylin.statusmanager.interface", "/", "com.kylin.statusmanager.interface", "get_current_rotation");
        QDBusMessage response2 = QDBusConnection::sessionBus().call(rotationMsg);
        if (response2.type() == QDBusMessage::ReplyMessage) {
            rotationState = response2.arguments().takeFirst().toString();
        } else {
            qCritical() << "get_current_rotation error:" << response.type();
        }
        //获取当前主屏幕信息
        m_iScreenXGeometry = m_pSreenInfo->m_nScreen_x;
        m_iScreenYGeometry = m_pSreenInfo->m_nScreen_y;
        m_iScreenWidth = m_pSreenInfo->m_screenWidth;
        m_iScreenHeight = m_pSreenInfo->m_screenHeight;
        //根据任务栏的位置、侧边栏位置和旋转状态决定通知弹窗的显示位置
        switch (m_ipanelPosition) {
            case topTransparentWidget::PanelDown:
                setWidgetPos(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin), m_iScreenYGeometry + INTER::topWidgeTopMargin);
                m_point.setX(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin));
                m_point.setY(m_iScreenYGeometry + INTER::topWidgeTopMargin);
                break;
            case topTransparentWidget::PanelUp:
                setWidgetPos(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin), m_iScreenYGeometry + INTER::topWidgeTopMargin + m_ipanelHeight); // 获取任务栏高度
                m_point.setX(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin));
                m_point.setY(m_iScreenYGeometry + INTER::topWidgeTopMargin + m_ipanelHeight);
                break;
            case topTransparentWidget::PanelLeft:
                setWidgetPos(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - INTER::topWidgeRightMargin), m_iScreenYGeometry + INTER::topWidgeTopMargin);
                m_point.setX(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - INTER::topWidgeRightMargin));
                m_point.setY(m_iScreenYGeometry + INTER::topWidgeTopMargin);
                break;
            case topTransparentWidget::PanelRight:
                setWidgetPos(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin - m_ipanelHeight), m_iScreenYGeometry + INTER::topWidgeTopMargin);
                m_point.setX(m_iScreenXGeometry + (m_iScreenWidth - INTER::topWidgetWidth - sidebarWidth - INTER::topWidgeRightMargin - m_ipanelHeight));
                m_point.setY(m_iScreenYGeometry + INTER::topWidgeTopMargin);
                break;
            default:
                break;
        }
    }
    return;
}

/* 判断popupWidget是否在topWidget中 */
bool topTransparentWidget::isPopupExist(uint replacesId)
{
    for (int i = 0; i < m_popWidgetqueue.size(); ++i) {
        popupItemWidget *popwidget = m_popWidgetqueue.at(i);
        if (popwidget->m_pentryInfo->id().toUInt() == replacesId)
            return true;
    }
    return false;
}

/* 重新绘制背景色 */
void topTransparentWidget::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    QPainterPath path;
    opt.rect.adjust(0, 0, 0, 0);
    path.addRoundedRect(opt.rect, 6, 6);
    p.setRenderHint(QPainter::Antialiasing); //反锯齿
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
    QWidget::paintEvent(event);
}

/* 开启PopWidget窗口隐藏动画 */
void topTransparentWidget::startPopWidgetHideAnimation(QWidget *w, int id)
{
    Q_UNUSED(id);
    popupItemWidget *popw = dynamic_cast<popupItemWidget *>(w);
    for (int i = 0; i < m_popWidgetqueue.count(); i++) {
        if (popw == m_popWidgetqueue.at(i)) {
            uint closeReason = popw->getCloseReason();
            if (closeReason == topTransparentWidget::TimeOut || closeReason == topTransparentWidget::ClosedByApp
                || closeReason == topTransparentWidget::Undefined) {
                popw->showToSidebar();
            }
            m_popWidgetqueue.at(i)->m_pOutAnimation->setStartValue(m_popWidgetqueue[i]->geometry());
            m_popWidgetqueue.at(i)->m_pOutAnimation->setEndValue(
                QRect(this->width() + 10, m_popWidgetqueue[i]->geometry().y(), this->width(), 88));
            m_popWidgetqueue.at(i)->m_pOutAnimation->start();
        }
    }
    return;
}

/* 隐藏动画完成槽函数 */
void topTransparentWidget::hideAnimationFinishSlots(QWidget *w, int id)
{
    popupItemWidget *popw = dynamic_cast<popupItemWidget *>(w);
    popw->hide();
    if (popw->m_pentryInfo->timeout() == "0") {
        --m_fixNotifyNum;
    }
    exitPopupWidget(w);
    if (!popw->getCloseActionKey().isEmpty()) {
        emit actionInvoked(id, popw->getCloseActionKey());
    } else {
        emit notificationClosed(id, popw->getCloseReason());
    }
    addWaittingPopupWidgetSlots();
    return;
}

/* 通过id关闭对应的弹窗窗口 */
void topTransparentWidget::moveAllpopWidgetSiteAccordId(int Id)
{
    for (int i = 0; i < m_popWidgetqueue.count(); i++) {
        popupItemWidget *popupWidget = m_popWidgetqueue.at(i);
        if (popupWidget->m_pentryInfo->id().toUInt() == Id) {
            /*
             * 容错处理：每个弹窗只允许应用方关闭一次。曾出现过应用多次关闭同一弹窗导致侧边栏里收纳了多个相同的弹窗的情況
             * 处理逻辑：判断popupWidget的m_closeReason标志位，如果不是ClosedReason::ClosedByApp则设置成ClosedByApp,
             *         ，如果关闭原因是ClosedByApp，则不做操作。
             * 备注：这里不对m_popWidgetqueue加锁的原因是，从应用发出删除信号开始到删除动画完成时间较长，
             *      这段时间里可能会有其他通知会做入队出队操作。
            */
            if (popupWidget->getCloseReason() != ClosedReason::ClosedByApp) {
                popupWidget->setCloseReason(ClosedReason::ClosedByApp);

                popupWidget->showToSidebar();
                m_popWidgetqueue.at(i)->m_pOutAnimation->setStartValue(m_popWidgetqueue[i]->geometry());
                m_popWidgetqueue.at(i)->m_pOutAnimation->setEndValue(
                    QRect(this->width() + 10, m_popWidgetqueue[i]->geometry().y(), this->width(), 88));
                m_popWidgetqueue.at(i)->m_pOutAnimation->start();
                break;
            } else {
                break;
            }
        }
    }
    return;
}

/* 毛玻璃效果 */
void topTransparentWidget::TransformGroundGlassAreaSlots(const QVariant &value, QWidget *w)
{
    if (m_popWidgetqueue.count() == 1) {
        this->setProperty("blurRegion", QRegion(QRect(0, 0, MASTER::topWidgetWidth, this->height() - w->height() + 1)));
    } else {
        this->setProperty("blurRegion", QRegion(QRect(0, 0, MASTER::topWidgetWidth, this->height() - w->height())));
    }
}

/* 将等待队列中的消息添加到topWidget中 */
void topTransparentWidget::addWaittingPopupWidgetSlots()
{
    if (m_pWaitingQueue.count() != 0) {
        AddPopupItemWidget(m_pWaitingQueue.takeAt(0));
    }
    return;
}

/* 任务栏位置或高度改变槽函数 */
void topTransparentWidget::panelSiteSlots(QString key)
{
    if (key == "panelposition" || key == "panelsize") {
        m_ipanelPosition = m_pPanelSetting->get("panelposition").toInt();
        m_ipanelHeight = m_pPanelSetting->get("panelsize").toInt();
        setNotifyPopWidgetSite();
    }
    return;
}

/* 通知与侧边栏联动动画
 * 参数：
 *      time:动画持续时间
 *      distance：移动距离
 */
void topTransparentWidget::animationActionSlots(const uint time, const int distance)
{
    /* 获取任务栏位置 */
    int panelPosition = 0;
    if (m_pPanelSetting && m_pPanelSetting->keys().contains("panelposition")) {
        panelPosition = m_pPanelSetting->get("panelposition").toInt();
    } else {
        return;
    }
    if (panelPosition != topTransparentWidget::PanelLeft) {
        QPoint startPoint;
        QPoint endPoint;
        startPoint = m_point;
        endPoint = m_point - QPoint(distance, 0);
        m_point = endPoint; //记录当前结束位置，作为下一次的开始位置
        m_posAnimation->stop();
        m_posAnimation->setDuration(time);
        m_posAnimation->setStartValue(startPoint);
        m_posAnimation->setEndValue(endPoint);
        m_posAnimation->start();
    }
}
